/*  minIni - Multi-Platform INI file parser, suitable for embedded systems
 *
 *  Copyright (c) CompuPhase, 2008-2021
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy
 *  of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations
 *  under the License.
 *
 *  Version: $Id: minIni.h 53 2015-01-18 13:35:11Z thiadmer.riemersma@gmail.com $
 */
#ifndef MININI_H
#define MININI_H

#include "minGlue.h"

#if (defined _UNICODE || defined __UNICODE__ || defined UNICODE) && !defined INI_ANSIONLY
  #include <tchar.h>
  #define mTCHAR TCHAR
#else
  /* force TCHAR to be "char", but only for minIni */
  #define mTCHAR char
#endif

#if !defined INI_BUFFERSIZE
  #define INI_BUFFERSIZE  512
#endif

#if defined __cplusplus
  extern "C" {
#endif

int   ini_getbool(const mTCHAR *Section, const mTCHAR *Key, int DefValue, const mTCHAR *Filename);
long  ini_getl(const mTCHAR *Section, const mTCHAR *Key, long DefValue, const mTCHAR *Filename);
int   ini_gets(const mTCHAR *Section, const mTCHAR *Key, const mTCHAR *DefValue, mTCHAR *Buffer, int BufferSize, const mTCHAR *Filename);
int   ini_getsection(int idx, mTCHAR *Buffer, int BufferSize, const mTCHAR *Filename);
int   ini_getkey(const mTCHAR *Section, int idx, mTCHAR *Buffer, int BufferSize, const mTCHAR *Filename);

int   ini_hassection(const mTCHAR *Section, const mTCHAR *Filename);
int   ini_haskey(const mTCHAR *Section, const mTCHAR *Key, const mTCHAR *Filename);

#if defined INI_REAL
INI_REAL ini_getf(const mTCHAR *Section, const mTCHAR *Key, INI_REAL DefValue, const mTCHAR *Filename);
#endif

#if !defined INI_READONLY
int   ini_putl(const mTCHAR *Section, const mTCHAR *Key, long Value, const mTCHAR *Filename);
int   ini_puts(const mTCHAR *Section, const mTCHAR *Key, const mTCHAR *Value, const mTCHAR *Filename);
#if defined INI_REAL
int   ini_putf(const mTCHAR *Section, const mTCHAR *Key, INI_REAL Value, const mTCHAR *Filename);
#endif
#endif /* INI_READONLY */

#if !defined INI_NOBROWSE
typedef int (*INI_CALLBACK)(const mTCHAR *Section, const mTCHAR *Key, const mTCHAR *Value, void *UserData);
int  ini_browse(INI_CALLBACK Callback, void *UserData, const mTCHAR *Filename);
#endif /* INI_NOBROWSE */

#if defined __cplusplus
  }
#endif


#if defined __cplusplus

#if defined __WXWINDOWS__
	#include "wxMinIni.h"
#else
  #include <string>

  /* The C++ class in minIni.h was contributed by Steven Van Ingelgem. */
#ifndef MININI_IMPLEMENTATION
  #define MININI_IMPLEMENTATION 1
#endif
  class minIni
  {
  public:
    minIni(const std::string& filename)
#if MININI_IMPLEMENTATION
      : iniFilename(filename)
       { }
#else
      ;
#endif

    bool getbool(const std::string& Section, const std::string& Key, bool DefValue=false) const
#if MININI_IMPLEMENTATION
      { return ini_getbool(Section.c_str(), Key.c_str(), int(DefValue), iniFilename.c_str()) != 0; }
#else
      ;
#endif

    long getl(const std::string& Section, const std::string& Key, long DefValue=0) const
#if MININI_IMPLEMENTATION
      { return ini_getl(Section.c_str(), Key.c_str(), DefValue, iniFilename.c_str()); }
#else
      ;
#endif

    int geti(const std::string& Section, const std::string& Key, int DefValue=0) const
#if MININI_IMPLEMENTATION
      { return static_cast<int>(this->getl(Section, Key, long(DefValue))); }
#else
      ;
#endif

    std::string gets(const std::string& Section, const std::string& Key, const std::string& DefValue="") const
#if MININI_IMPLEMENTATION
      {
        char buffer[INI_BUFFERSIZE];
        ini_gets(Section.c_str(), Key.c_str(), DefValue.c_str(), buffer, INI_BUFFERSIZE, iniFilename.c_str());
        return buffer;
      }
#else
      ;
#endif

    std::string getsection(int idx) const
#if MININI_IMPLEMENTATION
      {
        char buffer[INI_BUFFERSIZE];
        ini_getsection(idx, buffer, INI_BUFFERSIZE, iniFilename.c_str());
        return buffer;
      }
#else
      ;
#endif

    std::string getkey(const std::string& Section, int idx) const
#if MININI_IMPLEMENTATION
      {
        char buffer[INI_BUFFERSIZE];
        ini_getkey(Section.c_str(), idx, buffer, INI_BUFFERSIZE, iniFilename.c_str());
        return buffer;
      }
#else
      ;
#endif

    bool hassection(const std::string& Section) const
#if MININI_IMPLEMENTATION
      { return ini_hassection(Section.c_str(), iniFilename.c_str()) != 0; }
#else
      ;
#endif

    bool haskey(const std::string& Section, const std::string& Key) const
#if MININI_IMPLEMENTATION
      { return ini_haskey(Section.c_str(), Key.c_str(), iniFilename.c_str()) != 0; }
#else
      ;
#endif

#if defined INI_REAL
    INI_REAL getf(const std::string& Section, const std::string& Key, INI_REAL DefValue=0) const
#if MININI_IMPLEMENTATION
      { return ini_getf(Section.c_str(), Key.c_str(), DefValue, iniFilename.c_str()); }
#else
      ;
#endif
#endif

#if ! defined INI_READONLY
    bool put(const std::string& Section, const std::string& Key, long Value)
#if MININI_IMPLEMENTATION
      { return ini_putl(Section.c_str(), Key.c_str(), Value, iniFilename.c_str()) != 0; }
#else
      ;
#endif

    bool put(const std::string& Section, const std::string& Key, int Value)
#if MININI_IMPLEMENTATION
      { return ini_putl(Section.c_str(), Key.c_str(), (long)Value, iniFilename.c_str()) != 0; }
#else
      ;
#endif

    bool put(const std::string& Section, const std::string& Key, bool Value)
#if MININI_IMPLEMENTATION
      { return ini_putl(Section.c_str(), Key.c_str(), (long)Value, iniFilename.c_str()) != 0; }
#else
      ;
#endif

    bool put(const std::string& Section, const std::string& Key, const std::string& Value)
#if MININI_IMPLEMENTATION
      { return ini_puts(Section.c_str(), Key.c_str(), Value.c_str(), iniFilename.c_str()) != 0; }
#else
      ;
#endif

    bool put(const std::string& Section, const std::string& Key, const char* Value)
#if MININI_IMPLEMENTATION
      { return ini_puts(Section.c_str(), Key.c_str(), Value, iniFilename.c_str()) != 0; }
#else
      ;
#endif

#if defined INI_REAL
    bool put(const std::string& Section, const std::string& Key, INI_REAL Value)
#if MININI_IMPLEMENTATION
      { return ini_putf(Section.c_str(), Key.c_str(), Value, iniFilename.c_str()) != 0; }
#else
      ;
#endif
#endif

    bool del(const std::string& Section, const std::string& Key)
#if MININI_IMPLEMENTATION
      { return ini_puts(Section.c_str(), Key.c_str(), 0, iniFilename.c_str()) != 0; }
#else
      ;
#endif

    bool del(const std::string& Section)
#if MININI_IMPLEMENTATION
      { return ini_puts(Section.c_str(), 0, 0, iniFilename.c_str()) != 0; }
#else
      ;
#endif
#endif

#if !defined INI_NOBROWSE
    bool browse(INI_CALLBACK Callback, void *UserData) const
#if MININI_IMPLEMENTATION
      { return ini_browse(Callback, UserData, iniFilename.c_str()) != 0; }
#else
      ;
#endif
#endif

  private:
    std::string iniFilename;
  };

#endif /* __WXWINDOWS__ */
#endif /* __cplusplus */

#endif /* MININI_H */
